/***************************************************************************
 *   Copyright (C) 2007 by ASL-ETH                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QtGui>
#include <QMessageBox>
#include <QDir>
#include <QFileDialog>
#include <QByteArray>
#include <QChar>
#include <qwidget.h>
#include <QPrintDialog>
#include <QPrinter>
#include <QPixmap>
#include <QImage>
#include <content.h>
#include <unistd.h>
#include <QColor>
#include <time.h>

#ifdef APP_WIN32
	#include <windows.h>
#else
	#include <dirent.h>
#endif

#include <math.h>
#include <stdio.h>
#include <QSound>


using namespace std;

/**
This function is the main entrance for the program GUI. It creates all the interfaces and initializes the configuration.
*/
Dialog::Dialog()
{
    cout << "Hello ORTK 0.1!\n" << endl;

    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(timerOR()));

    //initialize menues right in the beginning
    createMenu();
    //textfield for outpu information
    bigEditor = new QTextEdit;
    bigEditor->append(tr("<b>Welcome!</b>"));
    //build layout
    QGridLayout *mainLayout = new QGridLayout;
    mainLayout->setMenuBar(menuBar);

    mainLayout->addWidget(bigEditor, 0, 1, 6, 1);
    mainLayout->addWidget(createFirstExclusiveGroup(), 0, 2, 1, 1);
    mainLayout->addWidget(createSecondExclusiveGroup(), 2, 2, 1, 1);
    mainLayout->addWidget(createThirdExclusiveGroup(), 3, 2, 1, 1);
    mainLayout->addWidget(createFourExclusiveGroup(), 0, 3, 1, 1);
    mainLayout->setColumnMinimumWidth(1, 400);
    setLayout(mainLayout);
    setWindowTitle(tr("ORTK 0.1"));

    k0=NULL, k1=NULL, k2=NULL;
    nbtp=0, nbmatch=0, nbmatchgood=0, nbfeatmodel=0;

    cap = 0;
    setValue(22);
    saveFrameState = 0;
    successMatch = 0;
    contFrameCap = 0;
    globalSaver = 0;

    ReadConfig();
}

/**
This function creates the TOP menu bar and displays all the buttons with asociated functions.
*/
void Dialog::createMenu()
{
    menuBar = new QMenuBar;

    ///First menu
    fileMenu = new QMenu(tr("&File"), this);
    //Save
    saveAction = fileMenu->addAction(tr("&Save"));
    connect(saveAction, SIGNAL(triggered()), this, SLOT(saveFile()));
    saveAction->setShortcut(tr("Ctrl+S"));
    //Print
    printAction = fileMenu->addAction(tr("&Print"));
    connect(printAction, SIGNAL(triggered()), this, SLOT(printFile()));
    printAction->setShortcut(tr("Ctrl+P"));
    fileMenu->addSeparator();
    //Exit
    exitAction = fileMenu->addAction(tr("E&xit"));
    exitAction->setShortcut(tr("Ctrl+Q"));
    connect(exitAction, SIGNAL(triggered()), this, SLOT(accept()));

    menuBar->addMenu(fileMenu);

    ///Second Menu
    helpMenu = new QMenu(tr("&Help"), this);
    //About
    aboutAction = new QAction(tr("About"), this);
    helpMenu->addAction(aboutAction);
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(about()));
    //Instructions
    Popup *obj = new Popup;
    helpAction = new QAction(tr("Instructions"), this);
    helpMenu->addAction(helpAction);
    connect(helpAction, SIGNAL(triggered()), obj, SLOT(exec()));

    menuBar->addMenu(helpMenu);
}

/**
This function creates the LEFT-TOP menu bar and displays all the buttons with asociated functions.
*/
QGroupBox *Dialog::createFirstExclusiveGroup()
{
    QGroupBox *groupBox = new QGroupBox(tr("360 Recognition"));
    comboBox = new QComboBox;
    char *dir3("training");
    DIR *d3 = opendir(dir3);
    struct dirent *dirent3;
    comboBox->addItem(tr("Load training images"));

    while((dirent3 = readdir(d3)) != NULL){
    	if(dirent3->d_name[0] != '.'){
    	comboBox->addItem(dirent3->d_name);
    	}
    }
    closedir(d3);

    connect(comboBox, SIGNAL(activated(int)),this, SLOT(trainImages()));

    browseButton1 = new QPushButton(tr("Browse 1"));
    connect(browseButton1, SIGNAL(clicked()), this, SLOT(open1()));
    browseButton1->setEnabled(false);
    runButton1 = new QPushButton(tr("Run Recognition 1"));
    runButton1->setEnabled(false);
    connect(runButton1, SIGNAL(clicked()), this, SLOT(Recognition()));

    int frameStyle = QFrame::Sunken | QFrame::Panel;
    textLabel = new QLabel;
    textLabel->setFrameStyle(frameStyle);
    textLabel->setFixedHeight(20);

    clusterLabel = new QLabel;
    clusterLabel->setFixedHeight(20);

    slider = new QSlider( Qt::Horizontal, this);
    slider->setFocusPolicy(Qt::StrongFocus);
    slider->setTickPosition(QSlider::TicksBothSides);
    slider->setTickInterval(10);
    slider->setSingleStep(1);
    connect( slider, SIGNAL( valueChanged( int ) ), this, SLOT( setValue(int) ) );

    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addWidget(comboBox);
    vbox->addWidget(textLabel);
    vbox->addWidget(browseButton1);
    vbox->addWidget(runButton1);
    vbox->addWidget(clusterLabel);
    vbox->addWidget(slider);

    vbox->addStretch(1);
    groupBox->setLayout(vbox);
    return groupBox;
}

/**
This function creates the LEFT-MIDDLE menu bar and displays all the buttons with asociated functions.

*/
QGroupBox *Dialog::createSecondExclusiveGroup()
{
    simpleRecText = new QLabel(this);
    simpleRecText->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    simpleRecText->setFixedHeight(20);

    QGroupBox *groupBox = new QGroupBox(tr("Simple Recognition"));
    browseButton2 = new QPushButton(tr("Browse 2"));
    connect(browseButton2, SIGNAL(clicked()), this, SLOT(open2()));
    runButton2 = new QPushButton(tr("Run Recognition 2"));
    runButton2->setEnabled(false);
    connect(runButton2, SIGNAL(clicked()), this, SLOT(receiver()));

    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addWidget(simpleRecText);
    vbox->addWidget(browseButton2);
    vbox->addWidget(runButton2);

    vbox->addStretch(1);
    groupBox->setLayout(vbox);
    return groupBox;
}

/**
This function creates the LEFT-BOTTOM menu bar and displays all the buttons with asociated functions.

MOVE TO GUI
*/
QGroupBox *Dialog::createThirdExclusiveGroup()
{
    QGroupBox *groupBox = new QGroupBox(tr("Capture"));

    openCamera = new QPushButton(tr("Open Camera"));
    connect(openCamera, SIGNAL(clicked()), this, SLOT(CaptureControl()));
    singleFrame = new QPushButton(tr("Single Frame OR"));
    connect(singleFrame, SIGNAL(clicked()), this, SLOT(SingleFrameCap()));
    singleFrame->setEnabled(false);
    continuousFrame = new QPushButton(tr("Continuous Frame OR"));
    connect(continuousFrame, SIGNAL(clicked()), this, SLOT(ContFrameCap()));
    continuousFrame->setEnabled(true);
    saveFrame = new QPushButton(tr("Save Frame as Template"));
    saveFrame->setEnabled(false);
    connect(saveFrame, SIGNAL(clicked()), this, SLOT(SaveFrame()));

    captureTiming = new QLabel;
    captureTiming->setFixedSize(130,20);
    QVBoxLayout *vbox = new QVBoxLayout;

    vbox->addWidget(openCamera);
    vbox->addWidget(saveFrame);
    vbox->addWidget(singleFrame);
    vbox->addWidget(continuousFrame);

    vbox->addStretch(1);
    groupBox->setLayout(vbox);
    return groupBox;
}

/**
This function creates the RIGHT menu bar and displays all the buttons with asociated functions.
*/
QGroupBox *Dialog::createFourExclusiveGroup()
{
    QGroupBox *groupBox = new QGroupBox(tr("General"));
    configButton = new QPushButton(tr("Configuration"));
    connect(configButton, SIGNAL(clicked()), this, SLOT(ReadConfig()));

    Popup *obj = new Popup;
    saveButton = new QPushButton(tr("Save Text"));
    saveButton->setFixedSize(130,25);
    connect(saveButton, SIGNAL(clicked()), this, SLOT(saveFile()));
    cancelButton = new QPushButton(tr("Quit"));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(Quit()));
    helpButton = new QPushButton(tr("Instructions"));
    connect(helpButton, SIGNAL(clicked()), obj, SLOT(exec()));
    clearButton = new QPushButton(tr("Clear Text"));
    connect(clearButton, SIGNAL(clicked()), this, SLOT(clearClicked()));
    deleteTemplates = new QPushButton(tr("Delete Templates"));
    connect(deleteTemplates, SIGNAL(clicked()), this, SLOT(DeleteTemplates()));

    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addWidget(configButton);
    vbox->addWidget(helpButton);
    vbox->addWidget(saveButton);
    vbox->addWidget(clearButton);
    vbox->addWidget(deleteTemplates);
    vbox->addWidget(cancelButton);
    vbox->addStretch(1);
    groupBox->setLayout(vbox);
    return groupBox;
}

/**
This function removes from the disk all the PGM files found in the templates directory.
*/
void Dialog::DeleteTemplates()
{
    if(system("del templates\\*.pgm")){
        bigEditor->append(tr("ERROR: Deleting templates Folder!"));
    }else{
        bigEditor->append(tr("Deleting templates Folder OK"));
    }
}

/**
This function closes everything and exists the program.
*/
void Dialog::Quit()
{
    cvDestroyWindow( "Capture INPUT" );
    cvDestroyWindow( "Capture OUTPUT" );
    cvDestroyWindow( "OR Input-Output" );
    reject();
}

/**
This function is used the vary the threshold used in clustering.
*/
void Dialog::setValue(int i)
{
    slider->blockSignals( true );
    slider->setValue( i );
    slider->blockSignals( false );
    clusterVar = (i/10.3)+0.1;
    clusterLabel->setText(tr("Clustering: Thresh = %1").arg(clusterVar));
}

/**
This function changes the GUI if the camera capture is started or stopped.
*/
void Dialog::CaptureControl()
{
    if(cap==0){
        cap=1;
        comboBox->setEnabled(false);
        browseButton1->setEnabled(false);
        browseButton2->setEnabled(false);
        openCamera->setText("Close Camera");
        singleFrame->setEnabled(true);
        saveFrame->setEnabled(true);
        runButton1->setEnabled(false);
        runButton2->setEnabled(false);
        cancelButton->setEnabled(false);
        Capture();
    }else{
        cap=0;
        comboBox->setEnabled(true);
        singleFrame->setEnabled(false);
        saveFrame->setEnabled(false);
        openCamera->setText("Open Camera");
        browseButton1->setEnabled(true);
        browseButton2->setEnabled(true);
        cancelButton->setEnabled(true);
    }
}

/**
This Function converts a frame and then saves the original image (?) finally, it extracts the points of the new saved image.
*/
void Dialog::saveFrameStateFunction(IplImage *frame)
{
        saveFrameState=0;
        CvSize s;
        s.width=frame->width;
        s.height=frame->height;
        cvShowImage("Capture OUTPUT", frame);
        dest = cvCreateImage(s, 8, 1);
        cvConvertImage(frame, dest, CV_CVTIMG_FLIP);

#ifdef APP_WIN32
        GetSystemTime(&st);
#else
	time_t ltime=time(NULL);
	st=localtime(&ltime);
#endif

	char output[50];
        char name[50];

#ifdef APP_WIN32
        sprintf(name, "single_%d_%d_%d_%d", st.wYear, st.wMonth, st.wDay, st.wMilliseconds);
#else
	sprintf(name, "single_%d_%d_%d_%d", st->tm_year, st->tm_mon, st->tm_mday, st->tm_sec);
	free(st);
#endif

        sprintf(output, "images/%s.jpg", name);

        cvSaveImage(output, frame);
        char buf[255];
#ifdef APP_WIN32
	makekeys(name, "images", "templates", 'w');
#else
	makekeys(name, "images", "templates", 'l');
#endif
        bigEditor->append(tr("output = %1").arg(output));
}

/**
This function saves a new frame of the webcam to disk and then extracts its keypoints.
*/
void Dialog::singContFrameOR(IplImage *frame)
{
        singleFrameCap=0;
        if(remove("key/capture.key")){
            bigEditor->append(tr("Delete KEY failure!"));
        }
        if(remove("images/capture.pgm")){
            bigEditor->append(tr("Delete PGM failure!"));
        }
        cvShowImage("Capture OUTPUT", frame);

        char a[50]=("capture");
        char b[50];
        sprintf(b, "images/%s.jpg", a);
        cvSaveImage(b, frame);
        sprintf(fileName0, "%s", a);
        char name[50]=("capture");
        sprintf(fileName0, "%s", name);
#ifdef APP_WIN32
	makekeys(name, "images", "images",'w');
#else
	makekeys(name, "images", "images",'l');
#endif
        receiver();
}

/**
This function contains the initialization of camera capture and the main loop where images are continuously captured and processed.
*/
void Dialog::Capture()
{
    CvCapture* capture2 = cvCaptureFromCAM( CV_CAP_ANY );
    if(!capture2) {
        bigEditor->append(tr("Error! Capturing not working properly - Check your Camera!"));
    }
    cvNamedWindow( "Capture INPUT", CV_WINDOW_AUTOSIZE );
    cvNamedWindow( "Capture OUTPUT", CV_WINDOW_AUTOSIZE );
    //first resize windows
    cvResizeWindow("Capture OUTPUT", 352, 288);

    while(cap==1) {
        IplImage* frame = cvQueryFrame(capture2);
        if(!frame) {
            bigEditor->append(tr("Error! Frames not captured properly!"));
            break;
        }
        cvShowImage("Capture INPUT", frame);
        //save single frame as template
        if(saveFrameState==1){
            saveFrameStateFunction(frame);
        }
        //single/continuous frame OR
        if(singleFrameCap==1){
            singContFrameOR(frame);
        }
        //If ESC key pressed, Key=0x10001B under OpenCV 0.9.7(linux version), remove higher bits using AND operator
        if( (cvWaitKey(10) & 255) == 27 ){
                break;
        }
    }
    cvReleaseCapture( &capture2 );
    cvDestroyWindow( "Capture INPUT" );
    cvDestroyWindow( "Capture OUTPUT" );
}

void Dialog::SaveFrame()
{
    saveFrameState = 1;
}

void Dialog::SingleFrameCap()
{
        singleFrameCap = 1;
}

void Dialog::ContFrameCap()
{
    if(contFrameCap==0){
        contFrameCap=1;
        continuousFrame->setText("Stopp");
        openCamera->setEnabled(false);
        saveFrame->setEnabled(false);
        singleFrame->setEnabled(false);
        singleFrameCap = 1;
    }else{
        contFrameCap=0;
        timer->stop();
        openCamera->setEnabled(true);
        saveFrame->setEnabled(true);
        singleFrame->setEnabled(true);
        continuousFrame->setText("Continuous Frame OR");
    }
}

void Dialog::timerOR()
{
    singleFrameCap=1;
    timer->stop();
}

/**
Print the text of the application console.
*/
void Dialog::printFile()
{
    QTextDocument *document = bigEditor->document();

    QPrinter printer;

    QPrintDialog *dialog = new QPrintDialog(&printer, this);
    dialog->setWindowTitle(tr("Print Document"));
    if (dialog->exec() != QDialog::Accepted)
        return;
    document->print(&printer);
}

/**
Save the text of the application console to a file.
*/
void Dialog::saveFile()
{
    QString format = "txt";
    QString initialPath = QDir::currentPath() + tr("/untitled.") + format;
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save file as"), initialPath, "*.txt");

    if (!fileName.isEmpty()) {
        QFile file(fileName);
        QTextStream stream(&file);

        if (file.open(QFile::WriteOnly | QFile::Text)) {
            stream << bigEditor->toPlainText();
            file.close();
        }
    }
}

/**
This function extracts the keys of the file stored in 'fileName0', then loads the image and the newly extracted points.
*/
void Dialog::receiver()
{
    globalSaver=0;
    char buf[255];
#ifdef APP_WIN32
	makekeys(fileName0, "images", "images",'w');
#else
	makekeys(fileName0, "images", "images",'l');
#endif
	
    simpleRecText->setText(tr("%1").arg(fileName0));
    bigEditor->append(tr(""));
    bigEditor->append(tr("fileName0 = %1").arg(fileName0));
    bigEditor->append(tr(""));
    // Load images and keypoints files
    sprintf(buf,"images/%s.pgm",fileName0);
    im0=cvLoadImage(buf, 0);

    sprintf(buf,"key/%s.key", fileName0);
    k0 = ReadKeyFile(buf, NULL, 0, klistEmpty);

    /**
    Then reads the images and keypoint files of the training data.
    */
    //Load images from templates folder. Initialization of vars
    char fileName[10][100];
    Keypoint k[10];                // k1,k2,k3,..
    IplImage *im[10];              // im1, im2, im3,...
    int nbfeatmodel[10];
    numb = 0;

    char *directory("templates");
    d2 = opendir(directory);
    struct dirent *dirent2;

    while((dirent2 = readdir(d2)) != NULL){
            //exclude ".", ".."
            if(strcmp(dirent2->d_name, ".") && strcmp(dirent2->d_name, "..") && dirent2->d_name[0]!='.') { //strcmp() greater than 0 means ''str1'' is greater than ''str2''
                    sprintf(full_name, "%s/%s", directory, dirent2->d_name);
                    sprintf(name,"%s",dirent2->d_name);
                    //cut off filename extension
#ifdef APP_WIN32
			name[dirent2->d_namlen-4]='\0';
#else
			name[strlen(dirent2->d_name)-4]='\0';
#endif
                    sprintf(fileName[numb],"%s",name);
                    bigEditor->append(tr("Filename in templates Folder = %1").arg(fileName[numb]));

                    // Build keypoint files with make
#ifdef APP_WIN32
			makekeys(fileName[numb],"templates", "images", 'w');
#else
			makekeys(fileName[numb],"templates", "images", 'l');
#endif

                    // Load images and keypoint files
                    sprintf(buf,"templates/%s.pgm",fileName[numb]);
                    im[numb] = cvLoadImage(buf, 0);

                    sprintf(buf,"key/%s.key",fileName[numb]);
                    k[numb] = ReadKeyFile(buf, &nbfeatmodel[numb], 0, klistEmpty);
                    numb++;
            }
    }
    bigEditor->append(tr("You have chosen to match %1 against %2 tamplate(s)").arg(fileName0).arg(numb));

    if(im0 == NULL || k0 == NULL){
        bigEditor->append(tr("Command line does not specify all images and keys."));
    }else{
            //file output
            char buffer[100];
            sprintf(buffer,"out/%s.txt", fileName0);
            sprintf(globalFilename,"%s", fileName0);

            fp = fopen(buffer,"w");
            if (fp == NULL){
                printf("Cannot open output file");
                exit(1);
            }
            //Find matches from the 2 lists of keypoints - Check for different models inside the scene
            bigEditor->append(tr(""));
            bigEditor->append(tr("######## <b>Entering FindMatchesWithHT()</b> ########"));
            int n;
/**
The FindMatchesWithHT function is called.
*/
            for(n=0;n<numb;n++){
                    fprintf(fp,"#####################\n");
                    fprintf(fp,"Checking for Template: %d (searching for '%s' in '%s')\n", n+1, fileName[n], fileName0);
                    fprintf(fp,"#####################\n\n");
                    bigEditor->append(tr(""));
                    bigEditor->append(tr("########"));
                    bigEditor->append(tr("Template %1").arg(n+1));
                    bigEditor->append(tr("########"));
                    //find matches
                    FindMatchesWithHT(im[n]->width, im[n]->height, k[n], im0->width, im0->height, k0, &nbmatch, im[n], im0, n, nbfeatmodel[n]);
            }
/**
If the matching is successful, the image is displayed and saved in the out directory, otherwise the continuous frame capture is activated if required.
*/
            if(successMatch==1){
                successMatch=0;
                QSound success("sounds/chimes.wav");
                success.play();
                char tmpImg[50];
                sprintf(tmpImg, "out/%s.jpg", fileName0);
                DisplayImage(tmpImg);
            }else{
                QSound negative("sounds/sound.wav");
                negative.play();
                //do continuous frame capturing
                if(contFrameCap==1){
                    timer->start(contCaptureTimer);
                }
            }
            fclose(fp);
    }
}

/**
Function to read the training images.
*/
void Dialog::open1()
{
    fileNameFull = QFileDialog::getOpenFileName(this, tr("Open File"), QDir::currentPath());
    if (!fileNameFull.isEmpty()) {
        QImage image(fileNameFull);
        if (image.isNull()) {
            QMessageBox::information(this, tr("ORTK 0.1"), tr("Cannot load %1.").arg(fileNameFull));
            return;
        }
        char *mychar;
        char *aa;

        //get filename
        QFileInfo fi(fileNameFull);
        QString name = fi.fileName();

        if(nbtp>0){
            runButton1->setEnabled(true);

            char tmpImg[50];
            aa = name.toAscii().data();
            sprintf(tmpImg, "images\\%s", aa);
            DisplayImage(tmpImg);
        }else{
            textLabel->setText("");
            bigEditor->append(tr("<font color='red'>Load training images first</font>"));
        }

        name.remove(QRegExp(".pgm"));
        name.remove(QRegExp(".jpg"));
        name.remove(QRegExp(".PGM"));
        name.remove(QRegExp(".JPG"));
        mychar = name.toAscii().data();
        sprintf(fileName0_1, mychar);

        QString format = "jpg";
        QString initialPath = QDir::currentPath() + tr("/untitled.") + format;
        QString fileName = QDir::currentPath() + tr("/images/") + name + tr(".") + format;

        //bigEditor->append(fileName);
        if (!fileName.isEmpty()){
            image.save(fileName, format.toAscii());
        }
        textLabel->setText(fileName0_1);
    }
}

/**
Function to read the test images.
*/
void Dialog::open2()
{
    fileNameFull = QFileDialog::getOpenFileName(this, tr("Open File"), QDir::currentPath());
    if (!fileNameFull.isEmpty()) {
        QImage image(fileNameFull);
        if (image.isNull()) {
            QMessageBox::information(this, tr("ORTK 0.1"), tr("Cannot load %1.").arg(fileNameFull));
            return;
        }
        char *mychar;
        char *aa;
        //get filename
        QFileInfo fi(fileNameFull);
        QString name = fi.fileName();

        char tmpImg[50];

        aa = name.toAscii().data();
        sprintf(tmpImg, "images\\%s", aa);
        DisplayImage(tmpImg);

        name.remove(QRegExp(".pgm"));
        name.remove(QRegExp(".jpg"));
        name.remove(QRegExp(".PGM"));
        name.remove(QRegExp(".JPG"));
        mychar = name.toAscii().data();
        sprintf(fileName0, mychar);

        QString format = "jpg";
        QString initialPath = QDir::currentPath() + tr("/untitled.") + format;
        QString fileName = QDir::currentPath() + tr("/images/") + name + tr(".") + format;

        if (!fileName.isEmpty()){
            image.save(fileName, format.toAscii());
        }
        simpleRecText->setText(fileName0);
        runButton2->setEnabled(true);
    }
}

/**
Function to read and display a test image.
*/
void Dialog::DisplayImage(char *img)
{
    CvSize s;
    s.width=1;
    s.height=1;
    tempImg=cvCreateImage(s, 8, 3);
    imOrig=cvLoadImage(img, 1);

    
   cvNamedWindow("OR Input-Output", CV_WINDOW_AUTOSIZE );
   cvShowImage("OR Input-Output", imOrig);
    //do continuous frame capturing
    if(contFrameCap==1){
        timer->start(contCaptureTimer);
    }
}

/**
Function to read the model images.
*/
void Dialog::trainImages()
{

    bigEditor->append(tr("\n"));
    long nbfeattot=0;
    nbtp=0;
    LoadTemplates(klist, imlist, &nbtp, &nbfeattot);
}

/**
This function adds the training images set selected by the combo box to imlist and the associated keypoints to klist.
*/
void Dialog::LoadTemplates(Keypoint *klist, IplImage **imlist, int *pnbtp, long *pnbfeattot)
{
    bigEditor->append(tr("<b>####### Entering LoadTemplates() ########</b>"));
    bigEditor->append(tr(""));
    //set variables
    Keypoint kp1;
    char full_name[50], buf[256], name[50];
    IplImage* im;
    int i, nbmatch, bestnbmatch, nbmatchgood, nbim=0, initializer=0, bestid;
    Match * tabmatch, * besttabmatch;
    double err;
    long nbfeattot=0;
    Matrice param(4,1,0), param1;
    float thres;

    char fileTemp[100];
    QString fileName = comboBox->currentText();
    if(fileName!="Load Training images"){
        char *mychar2 = fileName.toAscii().data();
        sprintf(fileTemp, "training/%s", mychar2);
        bigEditor->append(fileTemp);
    }else{
        bigEditor->append(fileTemp);
    }
    char *dir(fileTemp);
    DIR *d = opendir(dir);
    struct dirent *dirent;

    totalnbviews=0;
    //For all templates in directory
    while((dirent = readdir(d)) != NULL){
            //exclude ".", ".."
            if(strcmp(dirent->d_name, ".") && strcmp(dirent->d_name, "..") && dirent->d_name[0]!='.') { //strcmp() greater than 0 means ''str1'' is greater than ''str2''
                    totalnbviews++;
                    sprintf(full_name, "%s/%s", dir, dirent->d_name);
                    sprintf(name,"%s",dirent->d_name);
                    //cut off filename extension
#ifdef APP_WIN32
                    name[dirent->d_namlen-4]='\0';
#else
		    name[strlen(dirent->d_name)-4]='\0';
#endif
		    bigEditor->append(tr("_____________________________________________"));

                    //get degrees of rotation from filename (first 3 characters)
                    int degree;
                    int x1=100,x2=10,x3=1;
                    x1 *= name[0]-'0';
                    x2 *= name[1]-'0';
                    x3 *= name[2]-'0';
                    degree = x1+x2+x3;

                    //make openCV image from template
                    im=cvLoadImage(full_name, 0);
                    if(im==NULL){
                            bigEditor->append(tr("Error allocating memory for image while loading templates"));
                            exit(1);
                    }
                    //Call makekeyfile to build keyfile
#ifdef APP_WIN32
			makekeys(name, fileTemp, "images", 'w');
#else
			makekeys(name, fileTemp, "images", 'l');
#endif

                    //Read features
                    sprintf(buf,"key/%s.key",name);
                    int nbfeat=0;
                    kp1 = ReadKeyFile(buf, &nbfeat, nbim, klistEmpty);
                    kp1->next2 = NULL;
                    kp1->degree=degree;

                    nbfeattot+=nbfeat;
                    if(kp1==NULL){
                            //bigEditor->append(tr("Error reading keyfile %1").arg(buf));
                            exit(1);
                    }
                    if(initializer==0){
                            sprintf(kp1->name, name);
                            klist[0]=kp1;
                            imlist[0]=im;
                            initializer = 1;
                    }

                    /**
                    Each new image read from a template collection is compared to all the previous template to form clusters.
                    */
                    //match with all previous views
                    bestnbmatch=0;
                    besttabmatch=NULL;
                    err=1e99;
                    bestid = 1000;
                    i=0;
                    for(i=0;i<nbtp;i++){
                            //avoid comparing 2 identical images
                            if(strcmp(name,klist[i]->name)){
                                    bigEditor->append(tr("Compare %1 with model view %2").arg(name).arg(klist[i]->name));
                                    /**
                                    Tester function is called to decide with which previous image the new image should be clustered. This function returns the bin of the hough transform with the highest number of matches.
                                    */
                                    tabmatch = Tester(im->width, im->height, kp1, imlist[i]->width, imlist[i]->height, klist[i], &nbmatch, NULL);
                                     //check for the number of maxium matches and store bestmatching view
                                     if(nbmatch>bestnbmatch){
                                            besttabmatch=tabmatch;
                                            bestnbmatch=nbmatch;
                                            bestid=i;
                                    }
                            }
                    }
                    //compute error using ST
                     if(bestnbmatch>=3){ //if the best possible match is greater than 3 (minimum number of matches to estimate affine transformation)
                            bigEditor->append(tr("Best image is = %1").arg(klist[bestid]->name));
/**
Once the best matching function is found, the similarity transformation relating the two images is computed and also the expected error.
*/
                            param=SimilarityParamApprox(besttabmatch, bestnbmatch,MAXOF2(imlist[bestid]->width, imlist[bestid]->height), &err, &nbmatchgood);
                            thres=MAXOF2(im->width,im->height) * clustering_threshold;
                            thres=thres/(err+clusterVar); //constant small means less model views
                            bigEditor->append(tr("Error e = %1 and Thres = %2").arg(err).arg(thres));
                    }else{
                            bigEditor->append(tr("There are too few matches for ST!"));
                    }
                    //compare error to treshhold
                    if(err<=thres){
                            /// If the error is small: append current view to found model view.
                            sprintf(kp1->name, name);
                            kp1->im=im;
                            kp1->nbim=totalnbviews;
                            kp1->next2 = klist[bestid]->next2;
                            klist[bestid]->next2=kp1;
                            bigEditor->append(tr("<font color='red'>now append child %1 to view %2</font>").arg(name).arg(klist[bestid]->name));
                    }else{
                            /// If the error es big: the new image is used as a new model view.
                            sprintf(kp1->name, name);
                            kp1->nbim=totalnbviews;
                            klist[nbtp]=kp1;
                            imlist[nbtp]=im;
                            nbtp++;
                    }
            }//end if
    }//end while

/// [COMMENT: a "viewhierarchy" function could be written in order to have a clear idea of how are the training images organized. Additionally this function could be able to show the training images. END COMMENT]

    //file output 1
    char buffer[100];
    sprintf(buffer,"out/training_views_%s.txt", klist[0]->name);
    fp = fopen(buffer,"w");
    if (fp == NULL){
            printf("Cannot open output file");
            exit(1);
    }
    bigEditor->append(tr("\n\nNumber of model views = %1").arg(nbtp));
    fprintf(fp, "Number of model views = %d\n\n", nbtp);
    int x;
    Keypoint k;
    for(x=0;x<nbtp;x++){
            bigEditor->append(tr("########## model view, name = %1 ##########").arg(klist[x]->name));
            fprintf(fp, "mainlist model view = %s\n", klist[x]->name);
            for (k=klist[x]->next2; k != NULL; k = k->next2) {
                    bigEditor->append(tr("name = %1").arg(k->name));
                    fprintf(fp, "childname = %s\n", k->name);
            }
            fprintf(fp, "\n");
    }
    browseButton1->setEnabled(true);
    fclose(fp);
    closedir(d);
}

/**
This function is used to decide if a model image should be considered a new "root" or be assigned as a "child" to an existing "root"
*/
Match *Dialog::Tester(int width1, int height1, Keypoint keys1, int width2, int height2, Keypoint keys2, int *pnbmatch, int *pnbfeattot)
{
    Keypoint k=NULL, match, kmax=NULL, rtabfirst[ThetaDivMax][ScaleDivMax][XDivMax][YDivMax];
    int rtabnb[ThetaDivMax][ScaleDivMax][XDivMax][YDivMax];
    Match * tabmatch;
    float tmpX, tmpY, alpha;
    float cx, cy; // Center of first image
    int i,j,l,m,dOri,dScale,dX,dY,nbmax=0,count2;
    tabmatch=NULL;
    cx=width1/2;
    cy=height1/2;

    // Initialize table
    for(i=0;i<ThetaDivMax;i++)
    for(j=0;j<ScaleDivMax;j++)
    for(l=0;l<XDivMax;l++)
    for(m=0;m<YDivMax;m++){
        rtabfirst[i][j][l][m]=NULL;
        rtabnb[i][j][l][m]=0;
    }
    // Match the keys in list keys1 to their best matches in keys2
    count1=0;
    int count3=0;

    /// For this the two images are compared using the Hough transform, and the best bin is returned.
    for (k= keys1; k != NULL; k = k->next) {
        count3++;
        match = CheckForMatch(k, keys2, distance_ratio);

        // If we have a match, perform double check
        if (match != NULL && IsMatch(match, k, keys1, distance_ratio)) {
            k->match=match;
            k->match->flag=1;
            k->flag=1;
            count1++;

            // Fill table of Hough Transform. first calculate table entries
            dOri=ROUND(((Set2Range2Pi(match->ori - k->ori))/2/PI*theta_div_HT))-1;
            dScale=ROUND((log((double)match->scale /(double)k->scale)/log(scale_factor))+ scale_div_HT/2)-1;

            // Compute angle between Kp-center vector and feature vector
            float normPO=sqrt(SQUARE(k->col-cx,k->row-cy));
            alpha=atan2(cy-k->row,cx-k->col)-k->ori;
            tmpX= match->col + (match->scale/k->scale)* cos(match->ori+alpha)*normPO;
            tmpY= match->row + (match->scale/k->scale)* sin(match->ori+alpha)*normPO;
            dX=ROUND((tmpX * x_div_HT) / width2)-1;
            dY=ROUND((tmpY * y_div_HT) / height2)-1;
            // Fill table of Hough transform

            i=j=l=m=0;
                int ii,jj,ll,mm;
                // Vote for the 2 best guess in each of the 4 dimension
                ii=(dOri+i+theta_div_HT)%theta_div_HT;
                if(ii>=theta_div_HT){cout<<"Error SDF4"<<endl;exit(1);}
                jj=dScale+j;
                ll=dX+l;
                mm=dY+m;

                if((jj>=0)&&(jj<scale_div_HT)&&(ll>=0)&&(ll<x_div_HT)&&(mm>=0)&&(mm<y_div_HT)){
                    //Add to chain list
                    Keypoint knew=Duplicate(k);
                    knew->next2=rtabfirst[ii][jj][ll][mm];
                    rtabfirst[ii][jj][ll][mm]=knew;
                    rtabnb[ii][jj][ll][mm]+=1;
                    //cout  <<"rtabnb["<<ii<<"]["<<jj<<"]["<<ll<<"]["<<mm<<"] rtabnb = "<<rtabnb[ii][jj][ll][mm]<<endl;
                    if(rtabnb[ii][jj][ll][mm]>nbmax)   // We keep the best bet
                    {
                        nbmax=rtabnb[ii][jj][ll][mm];
                        kmax=rtabfirst[ii][jj][ll][mm];
                    }
                }
        }
    }
    //Put all good matches in a table:
    tabmatch = (Match*) malloc (nbmax * sizeof(Match));
    count2=0;
    if (tabmatch==NULL){
        bigEditor->append(tr("Error allocating memory"));
        exit(1);
    }
    if(kmax!=NULL)
    for (k= kmax; k != NULL; k = k->next2) {
        if(count2>=nbmax){
            bigEditor->append(tr("Error while accessing tabmatch"));
            exit(1);
        }
        tabmatch[count2].x1=k->col;
        tabmatch[count2].y1=k->row;
        tabmatch[count2].ori1=k->ori;
        tabmatch[count2].scale1=k->scale;
        tabmatch[count2].x2=k->match->col;
        tabmatch[count2].y2=k->match->row;
        tabmatch[count2].ori2=k->match->ori;
        tabmatch[count2].scale2=k->match->scale;
        tabmatch[count2].id=1;
        count2++;
        k->match->flag=1;// Flag keypoint so that it cannot be used again
    }
    *pnbmatch=nbmax;
    bigEditor->append(tr("Found %1 matches and %2 coherent matches (after HT)").arg(count1).arg(count2));
    return tabmatch;
}

/**
This function extracts the keypoints of 'fileName0_1' and reads the image and the keypoints. Next
*/
void Dialog::Recognition()
{
    globalSaver=0;
    bigEditor->append(tr(""));
    bigEditor->append(tr(""));
    bigEditor->append(tr("<b>####### Entering Recognition() ########</b>"));
    char buf[100];

    // Build keypoint files
#ifdef APP_WIN32
	makekeys(fileName0_1, "images", "images", 'w');
#else
	makekeys(fileName0_1, "images", "images", 'l');
#endif

    // Load images and keypoints files
    sprintf(buf,"images/%s.pgm",fileName0_1);
    im0=cvLoadImage(buf, 0);

    sprintf(buf,"key/%s.key", fileName0_1);
    k0 = ReadKeyFile(buf, NULL, 0, klistEmpty);

    int bestnbmatch=0, bestid, bestnbChild1=0;
    char bestchildname[50];
    int bestnbChild2=0;
    Match *besttabmatch, *bestchildtabmatch;
    Keypoint w, bestkeypoint1, bestkeypoint2;
    IplImage *bestim;

/**
Tester is used again to compute the Hough transform between the new image and the learned models.
*/
    for(int i=0;i<nbtp;i++){
                // Compare the image with each model view
                nbmatchgood=0;
                bigEditor->append(tr("_____________________________________________"));
                bigEditor->append(tr("Compare %1 to model view %2 (degree=%3)").arg(fileName0_1).arg(klist[i]->name).arg(klist[i]->degree));
                tabmatch = Tester(imlist[i]->width, imlist[i]->height, klist[i], im0->width, im0->height, k0, &nbmatch, NULL);
                bigEditor->append(tr("--------------------------------------------------------------------"));
                if(nbmatch>bestnbmatch){
                            bestid=i;
                            bestnbmatch = nbmatch;
                            besttabmatch=tabmatch;
                }
    }
    bigEditor->append(tr(""));
    bigEditor->append(tr("##################"));
    bigEditor->append(tr("Best model view match = %1").arg(klist[bestid]->name));
    bigEditor->append(tr("##################"));

    nbmatch = 0;
/**
Once the best matching model is found the new image is compared to the model's children.
*/
    for (w=klist[bestid]->next2; w != NULL; w = w->next2) {
                bigEditor->append(tr("___________"));
                bigEditor->append(tr("Compare %1 to model view %2 (degree=%3)").arg(fileName0_1).arg(w->name).arg(w->degree));
                tabmatch = Tester(w->im->width, w->im->height, w, im0->width, im0->height, k0, &nbmatch, NULL);
                 if(nbmatch>bestnbChild1){
                        sprintf(bestchildname, w->name);
                        bestnbChild2 = bestnbChild1;
                        bestnbChild1 = nbmatch;
                        bestchildtabmatch = tabmatch;
                        bestkeypoint2 = bestkeypoint1;
                        bestkeypoint1=w;
                        bestim=w->im;
                }
                bigEditor->append(tr("------------------"));
    }

    bigEditor->append(tr(""));
    bigEditor->append(tr("bestnbChild1 = %1 and bestnbChild2 = %2").arg(bestnbChild1).arg(bestnbChild2));
    bigEditor->append(tr(""));

///If not enough coherent matches are found, a second run is made (?).
    if(bestnbChild1<nb_match_min_AT){
         int inc=0;
         bigEditor->append(tr(""));
         bigEditor->append(tr("Now comes second run because of too few coherent matches ------------>"));
         bigEditor->append(tr(""));
         for(int i=0;i<nbtp;i++){
             for (w=klist[inc]->next2; w != NULL; w = w->next2) {
                    bigEditor->append(tr("___________"));
                    bigEditor->append(tr("Compare %1 to model view %2 (degree=%3)").arg(fileName0_1).arg(w->name).arg(w->degree));
                    tabmatch = Tester(w->im->width, w->im->height, w, im0->width, im0->height, k0, &nbmatch, NULL);
                     if(nbmatch>bestnbChild1){
                            sprintf(bestchildname, w->name);
                            bestnbChild2 = bestnbChild1;
                            bestnbChild1 = nbmatch;
                            bestchildtabmatch = tabmatch;
                            bestkeypoint2 = bestkeypoint1;
                            bestkeypoint1=w;
                            bestim=w->im;
                    }
                    bigEditor->append(tr("------------------"));
            }
            inc++;
            bigEditor->append(tr("\n#####"));
        }
        bigEditor->append(tr(""));
        bigEditor->append(tr("bestnbChild1 = %1 and bestnbChild2 = %2").arg(bestnbChild1).arg(bestnbChild2));
        bigEditor->append(tr(""));
    }
    bigEditor->append(tr("<font color='red'>............................................................................</font>"));

    if(bestnbChild1>bestnbmatch){
            bigEditor->append(tr("Best matching view with %1 coherent matches is %2").arg(bestnbChild1).arg(bestchildname));
            sprintf(buf,"./out/%s_with_%s.jpg", fileName0_1, bestchildname);
    }else{
            bigEditor->append(tr("Best view is parent view with %1 coherent matches is %2").arg(bestnbmatch).arg(klist[bestid]->name));
            bestchildtabmatch = besttabmatch;
            bestnbChild1 = bestnbmatch;
            bestkeypoint1 = klist[bestid];
            bestim=imlist[bestid];
            sprintf(buf,"./out/%s_with_%s.jpg", fileName0_1, klist[bestid]->name);
    }
    bigEditor->append(tr("<font color='red'>............................................................................</font>"));
    //compute AT and use param1 to display matches
///An affine transformation is approximated using the best bin of the Hough transforms.

    Matrice param1;
    param1=AffineParamApprox(bestchildtabmatch, bestnbChild1, MAXOF2(bestim->width, bestim->height));
	if(bestnbChild1>=nb_match_min_AT){

            DisplayMatches(bestim, im0, bestchildtabmatch, bestnbChild1, param1, buf, bestkeypoint1, k0, 0, 1);
			

            bigEditor->append(tr("<b>Image has been saved successfully!</b>"));
            //make successfull output image visible
            DisplayImage(buf);
            //play sound
            QSound success("sounds/chimes.wav");
            success.play();
            //interpolate angle
            double degrees;

			if(bestnbChild2!=0)
			{
				bigEditor->append(tr("<b>Interpolating angle using %1*%2 degrees and %3*%4 degrees...</b>").arg(bestnbChild1).arg(bestkeypoint1->degree).arg(bestnbChild2).arg(bestkeypoint2->degree));
				degrees = (bestnbChild1*bestkeypoint1->degree+bestnbChild2*bestkeypoint2->degree)/(bestnbChild1+bestnbChild2);
			}
			else
			{
				degrees = (bestnbChild1*bestkeypoint1->degree)/bestnbChild1;
			}			
	bigEditor->append(tr("<b>Angle = %1 degrees</b>").arg(degrees));

    } else{
            QSound negative("sounds/sound.wav");
            negative.play();
            bigEditor->append(tr("<b>Too few coherent matches (%1) for image output</b>").arg(bestnbChild1));
    }
}

/**
This function matches the keypoints extracted from two images using the Hough transform to find correct hypothesis.
*/
Match *Dialog::FindMatchesWithHT(int width1, int height1, Keypoint keys1, int width2, int height2, Keypoint keys2, int *pnbmatch, IplImage *imN, IplImage *im0, int n, int nbfeatmodelN)
{

/* DEFINITION AND INICIALIZATION ***********************************************************/
    // keys2 belongs to im0
    Keypoint k=NULL, match, rtabfirst[ThetaDivMax][ScaleDivMax][XDivMax][YDivMax];
    IndexList *indexStart=NULL, *indexNew;
    int rtabnb[ThetaDivMax][ScaleDivMax][XDivMax][YDivMax];
    Match *tabmatch;
    float tmpX, tmpY, alpha;
    float cx, cy; // Center of first image
    int i,j,l,m,dOri,dScale,dX,dY,nbmax=0, count1,count2, count3=0;
    tabmatch=NULL;
    cx=width1/2;
    cy=height1/2;
    Matrice param, param1, param2;

    Matrice best_affine_ransac;

    char buf[200];

    // Initialize table
    for(i=0;i<ThetaDivMax;i++)
    for(j=0;j<ScaleDivMax;j++)
    for(l=0;l<XDivMax;l++)
    for(m=0;m<YDivMax;m++)
    {
        rtabfirst[i][j][l][m]=NULL;
        rtabnb[i][j][l][m]=0;
    }
/* ****************************************************************************************/


/* KEYPOINT MATCHING AND HOUGH TRANSFORM FILLING*******************************************/
    // Match the keys in list keys1 to their best matches in keys2
    count1=0;
    for (k= keys1; k != NULL; k = k->next)
    {
        count3++;
        match = CheckForMatch(k, keys2, distance_ratio);

        /// To enforce matches, double check is performed.
        if (match != NULL && IsMatch(match, k, keys1, distance_ratio))
        {
            k->match=match; //Attention 2. "match" here ist a keypoint struct
            count1++;

            // Fill table of Hough Transform - first calculate table entries
            dOri=ROUND(((Set2Range2Pi(match->ori - k->ori))/2/PI*theta_div_HT))-1;
            dScale=ROUND((log((double)match->scale /(double)k->scale)/log(scale_factor))+ scale_div_HT/2)-1;

            // Compute angle between Kp-center vector and feature vector
            float normPO=sqrt(SQUARE(k->col-cx,k->row-cy));
            alpha=atan2(cy-k->row,cx-k->col)-k->ori;
            tmpX= match->col + (match->scale/k->scale)* cos(match->ori+alpha)*normPO;
            tmpY= match->row + (match->scale/k->scale)* sin(match->ori+alpha)*normPO;
            dX=ROUND((tmpX * x_div_HT) / width2)-1;
            dY=ROUND((tmpY * y_div_HT) / height2)-1;

            // Fill table of Hough transform
            i=j=l=m=0;
            int ii,jj,ll,mm;
            // Vote for the 2 best guess in each of the 4 dimension
            ii=(dOri+i+theta_div_HT)%theta_div_HT;
            jj=dScale+j;
            ll=dX+l;
            mm=dY+m;

            if((jj>=0)&&(jj<scale_div_HT)&&(ll>=0)&&(ll<x_div_HT)&&(mm>=0)&&(mm<y_div_HT))
            {
                //Add to chain list
                Keypoint knew=Duplicate(k);
                knew->next2=rtabfirst[ii][jj][ll][mm];
                rtabfirst[ii][jj][ll][mm]=knew;

                rtabnb[ii][jj][ll][mm]+=1;
            }
        }
    }

/* ****************************************************************************************/


/// Once the Hough transform table is filled, it is scanned to build a IndexList with all the bins that have 3 or more matches.
    int counter=0;
    for(i=0;i<ThetaDivMax;i++)
    {
        for(j=0;j<ScaleDivMax;j++)
        {
            for(l=0;l<XDivMax;l++)
            {
                for(m=0;m<YDivMax;m++)
                {
                    if(rtabnb[i][j][l][m]>=3)
                    {
                        indexNew = (IndexList*) malloc (1* sizeof(IndexList));
                        indexNew->i1 = i;
                        indexNew->i2 = j;
                        indexNew->i3 = l;
                        indexNew->i4 = m;
                        indexNew->next = indexStart;
                        indexStart = indexNew;
                        counter++;
                    }
                }
            }
        }
    }

/* ****************************************************************************************/

/* AFFINE MODEL EXTRACTION (reweighted least squares can be used) *************************/

    //List to store centerpoints of hypothesis to avoid having multiple lying very close together
    CvPoint centerList[counter];
    int w;
    for(w=0;w<counter;w++)
    {
        centerList[w].x = 0;
        centerList[w].y = 0;
    }
    CvPoint *pcenterList;
    pcenterList = centerList;

    IndexList *mm;
    //Loop through the IndexList to find the arbitrary number of keypoint Lists
    int numberPic=0;

    for (mm= indexStart; mm != NULL; mm = mm->next)
    {
        bigEditor->append(tr("- - - - - - - - - - - - - - - - - - - -"));
        numberPic++;
        Keypoint keyTemp = rtabfirst[mm->i1][mm->i2][mm->i3][mm->i4];

        /// The best bins keypoints are taken to estimate the transformation.
        if(keyTemp!=NULL)
        {
            // Put all good matches in a table:
            tabmatch = (Match*) malloc (nbmax * sizeof(Match));
            count2=0;
            Keypoint k=NULL;
            for (k= keyTemp; k != NULL; k = k->next2)
            {
                tabmatch[count2].x1=k->col;
                tabmatch[count2].y1=k->row;
                tabmatch[count2].ori1=k->ori;
                tabmatch[count2].scale1=k->scale;
                tabmatch[count2].x2=k->match->col;
                tabmatch[count2].y2=k->match->row;
                tabmatch[count2].ori2=k->match->ori;
                tabmatch[count2].scale2=k->match->scale;
                tabmatch[count2].id=1;
                count2++;
                k->match->flag=1; //Flag keypoint so that it cannot be used again
            }
        }
        *pnbmatch=nbmax;

        bigEditor->append(tr("Found %1 and %2 coherent matches (after HT)").arg(count1).arg(count2));

        //RANSAC (Still not used)
        vector<int> inliers;
        float error;
        best_affine_ransac=RansacAffine(tabmatch,count2,inliers,error); /* A **/
	
	bigEditor->append(tr("Best affine by RANSAC:\n%f\t\t%f\t\t%f \n%f\t\t%f\t\t%f \n%f\t\t%f\t\t%f \n").arg(best_affine_ransac.get_val(0,0)).arg(best_affine_ransac.get_val(0,1)).arg(best_affine_ransac.get_val(0,2)).arg(best_affine_ransac.get_val(1,0)).arg(best_affine_ransac.get_val(1,1)).arg(best_affine_ransac.get_val(1,2)).arg(best_affine_ransac.get_val(2,0)).arg(best_affine_ransac.get_val(2,1)).arg(best_affine_ransac.get_val(2,2)));

        // get affine parameters from result table
        if(count2>=nb_match_min_AT)
        {
            param1=AffineParamApprox(tabmatch, count2, MAXOF2(im0->width, im0->height));
            sprintf(buf,"./out/%s.jpg", globalFilename);
            //Adding Template to the output textfile
            fprintf(fp,"%d. Detection successful with %d matches! Coordinates of bounding box (TL, TR, BR, BL):\n", numberPic, count2);

            DisplayMatches(imN, im0, tabmatch, count2, param1, buf, keys1, keys2, n, numberPic, pcenterList);

            if(count2>=nb_match_min_ST){
                bigEditor->append(tr("nbfeatmodelN = %1").arg(nbfeatmodelN));
                int nbmatchgood=0;
                param2=SimilarityParamApprox(tabmatch, count2, MAXOF2(im0->width, im0->height), NULL, &nbmatchgood);
            }
            else
            {
                bigEditor->append(tr("Not enough matches after min square verification."));
            }
        }else
        {
            bigEditor->append(tr("Not enough matches after Hough transform to compute affine transform."));
        }
    free(tabmatch);
    bigEditor->append(tr("-----------------------------------"));
    }
}

void Dialog::about()
{
    QMessageBox::about(this, tr("About SIFT"), tr("<p><b>SIFT (Scale Invariant Feature Transform)</b> is a method that transforms a \
    naively obtained set of features, obtained from the object to be recognized, into a robust feature set that incorporates \
    invariance to scale and rotation changes and to a considerable extent deals with illumination changes and changes to \
    viewing direction as well. This feature transform has been found to be very effective for recognizing textured objects. \
    </p><p>The aim of this project is to improve the existing SIFT based object recognition system which has been developed in an \
    earlier semester project and to design a GUI (graphical user interface) that can be used by everyone, including those \
    without knowledge about the system. The system shall be enhanced by (1) performing multiple hypothesis detection - \
    detecting multiple occurrences of the same object in the image, (2) incorporating viewpoint invariance -detecting the \
    object from different directions / views and (3) by adding a graphical user interface for being able to easily run a \
    simple object recognition process rather than using the computers command console. </p><p>This work is to be understood as an \
    extension of what has already been described by my precursor student. It will therefore not explain any \
    details on feature extraction and Hough Transform itself but illustrate how the system has been extended and what kind \
    of problems have been encountered doing so.</p>"));
}

/**
This function approximates the affine transformation between the model and the test image.
*/
Matrice Dialog::AffineParamApprox(Match * tabmatch, int nbmatch, int maxdim, double *perr, int *pnbmatchgood)
{
    Matrice *a=NULL, *u=NULL ,b(6,6),param(6,1,0),p1(2,1),p2(2,1),q1(2,1),q2(2,1),empty(0,0,0);
    int i, j;
    int maxindex;
    float maxerror=1;
    //cout<<"(AT) Approx of affine param with "<<nbmatch<<" matches."<<endl;
    if(perr!=NULL)*perr=1e99;

    /* count good matches*/
    for(i=0;i<nbmatch;i++){tabmatch[i].id=1;}
    int nbmatchgood=nbmatch;

    while(maxerror>0){
        a=new Matrice(2*nbmatchgood,6);
        u=new Matrice(2*nbmatchgood,1);

        j=0;
        for(i=0;i<nbmatch;i++){
            if(tabmatch[i].id!=0){
                /* First create the matrix A and u */
                a->set_val(2*j,0,tabmatch[i].x1);
                a->set_val(2*j,1,tabmatch[i].y1);
                a->set_val(2*j,4,1);
                a->set_val(2*j+1,2,tabmatch[i].x1);
                a->set_val(2*j+1,3,tabmatch[i].y1);
                a->set_val(2*j+1,5,1);
                u->set_val(2*j,0,tabmatch[i].x2);
                u->set_val(2*j+1,0,tabmatch[i].y2);
                j++;
            }
        }
        if (nbmatchgood<nb_match_min_AT){
            if(pnbmatchgood!=NULL)*pnbmatchgood=0;
            return empty;
        }
        b=a->t()*(*a);
        //cout<<"Determinant of b ="<<b.det()<<endl;
        try
        {
            param = (b.inverse()*a->t())*(*u);
        }
        catch(Erreur &tmp)
        {
            //cout << tmp.cause() << " - " << tmp.explic() << endl;
        }
        maxerror=0;
        for (i=0;i<nbmatch;i++){
            if(tabmatch[i].id!=0){

                /* Compute transform of keypoint vector*/
                /// After computing the affine transformation matrix it is verified computing the error between the original points and the matched points according to the transformation.

                p1.set_val(0,0,tabmatch[i].x1);
                p1.set_val(1,0,tabmatch[i].y1);
                q1.set_val(0,0,tabmatch[i].x1+tabmatch[i].scale1*cos(tabmatch[i].ori1));
                q1.set_val(1,0,tabmatch[i].y1+tabmatch[i].scale1*sin(tabmatch[i].ori1));
                p2=Affine(p1,param);
                q2=Affine(p2,param);

                /*Error between transform and reel match*/

                float dx=tabmatch[i].x2-p2.get_val(0,0);
                float dy=tabmatch[i].y2-p2.get_val(1,0);
                p2=q2-p2;

                if(dx*dx + dy*dy>agree_pos_AT * agree_pos_AT * maxdim * maxdim){
                    float tmp=sqrt(dx*dx+dy*dy)/agree_pos_AT/maxdim;
                    if (tmp>maxerror){
                        maxerror=tmp;
                        maxindex=i;
                    }
                }
            }
        }//end for

        if(maxerror>0){
            tabmatch[maxindex].id=0;
            nbmatchgood-=1;
            if(nbmatchgood<nb_match_min_AT){
                if(pnbmatchgood!=NULL)*pnbmatchgood=0;
                return empty;
            }
        }
    }//end while

    Matrice * c=new Matrice(2*nbmatchgood,1);
    *c=((*a)*param)-(*u);
    float err=sqrt(2*c->sqnorm()/(2*nbmatchgood-6));
    delete(c);
    if (perr!=NULL)*perr=err;

    delete(a);
    delete(u);
    return param;
}

/**
This function displays the matches found.
*/
void Dialog::DisplayMatches(IplImage* im1, IplImage* im0, Match * tabmatch, int nbmatch, Matrice param, char *fileName, Keypoint k1, Keypoint k0, int n, int numberPic, CvPoint *pcenterList)
{
    CvSize s;
    CvPoint pt1, pt2, ptc;
    CvScalar color, ccolor;
    int i,j,im0null=0;

    successMatch = 1;

    if(im0==NULL){
        s.width=1;
        s.height=1;
        im0=cvCreateImage(s, 8, 1);
        im0null=1;
    }
    //check whether new image has to be generated or existing one in the out directory can be used
     if(globalSaver==1){
            char test[100];
            sprintf(test,"out/%s.jpg", globalFilename);
            imOrig=cvLoadImage(test, 0);
    }else{
            imOrig = im0;
    }
    s.width=imOrig->widthStep;
    s.height=imOrig->height;

    im=cvCreateImage(s, 8, 1);
    cvZero(im);
    imcol=cvCreateImage(s, 8, 3);
    cvZero(imcol);

    //Fill images pixel by pixel
    for (i = 0; i < imOrig->height; i++){
        for (j = 0; j < imOrig->width; j++){
            im->imageData[i*im->widthStep+j] = imOrig->imageData[i*imOrig->widthStep+j];
        }
    }
    //Copy image in colors
    for(i=0;i<imcol->imageSize;i++){
        imcol->imageData[i]=im->imageData[i/3];
    }
    //Display matches
    for(i=0;i<nbmatch;i++){
        pt2.x=ROUND(tabmatch[i].x2);
        pt2.y=ROUND(tabmatch[i].y2);
        if(tabmatch[i].id==0){
            color=CV_RGB(255,0,0); //red for discarded matches
        }else{
            color=CV_RGB(0,255,255); // color for coherent matches
        }
        cvCircle(imcol, pt2, 3, color, 1);
    }

    // compute belief in object recognition
    double val = -1 / (double(nbmatch)-2.0) ;
    double prob = exp(val);

    //Affine transform of the image border
    if(param.size_m()>0){
        Matrice p1(2,1), p2(2,1), p3(2,1), p4(2,1);

        p1.set_val(0,0,0);
        p1.set_val(1,0,0);
        p2.set_val(0,0,im1->width);
        p2.set_val(1,0,0);
        p3.set_val(0,0,im1->width);
        p3.set_val(1,0,im1->height);
        p4.set_val(0,0,0);
        p4.set_val(1,0,im1->height);

        p1=Transform(p1,param);
        p2=Transform(p2,param);
        p3=Transform(p3,param);
        p4=Transform(p4,param);

        int fx = 0,fy = 0;
        fx = int(p1.get_val(0,0)); fy = int(p1.get_val(1,0));
        fprintf(fp,"x = %d, y = %d\n",fx,fy);
        fx = int(p2.get_val(0,0)); fy = int(p2.get_val(1,0));
        fprintf(fp,"x = %d, y = %d\n",fx,fy);
        fx = int(p3.get_val(0,0)); fy = int(p3.get_val(1,0));
        fprintf(fp,"x = %d, y = %d\n",fx,fy);
        fx = int(p4.get_val(0,0)); fy = int(p4.get_val(1,0));
        fprintf(fp,"x = %d, y = %d\n",fx,fy);

        ptc.x = 0;
        ptc.y = 0;
        color=CV_RGB(0,128,255); //light blue

        pt1.x=ROUND(p1.get_val(0,0));
        pt1.y=ROUND(p1.get_val(1,0));
        pt2.x=ROUND(p2.get_val(0,0));
        pt2.y=ROUND(p2.get_val(1,0));
        cvLine(imcol, pt1, pt2, color, 1);

        ptc.x = ptc.x + pt1.x + pt2.x;
        ptc.y = ptc.y + pt1.y + pt2.y;

        pt1.x=ROUND(p2.get_val(0,0));
        pt1.y=ROUND(p2.get_val(1,0));
        pt2.x=ROUND(p3.get_val(0,0));
        pt2.y=ROUND(p3.get_val(1,0));
        cvLine(imcol, pt1, pt2, color, 1);

        ptc.x = ptc.x + pt1.x + pt2.x;
        ptc.y = ptc.y + pt1.y + pt2.y;

        pt1.x=ROUND(p3.get_val(0,0));
        pt1.y=ROUND(p3.get_val(1,0));
        pt2.x=ROUND(p4.get_val(0,0));
        pt2.y=ROUND(p4.get_val(1,0));
        cvLine(imcol, pt1, pt2, color, 1);

        ptc.x = ptc.x + pt1.x + pt2.x;
        ptc.y = ptc.y + pt1.y + pt2.y;

        pt1.x=ROUND(p4.get_val(0,0));
        pt1.y=ROUND(p4.get_val(1,0));
        pt2.x=ROUND(p1.get_val(0,0));
        pt2.y=ROUND(p1.get_val(1,0));
        cvLine(imcol, pt1, pt2, color, 1);

        ptc.x = ptc.x + pt1.x + pt2.x;
        ptc.y = ptc.y + pt1.y + pt2.y;

        ptc.x = ptc.x / 8;
        ptc.y = ptc.y / 8;

        //Check whether there are identical (x,y) centerpoints. If yes discard image.
        int saver=1;
        if(pcenterList!=NULL){
                int checker=0;
                int x;
                float sqrDist;

                while( (x = pcenterList[checker].x) ){
                        //compute distance between the two points. If distance < treshold then discard the image
                        sqrDist = sqrt(pcenterList[checker].x-ptc.x)+sqrt(pcenterList[checker].y-ptc.y);
                        if(ptc.x == pcenterList[checker].x && ptc.y == pcenterList[checker].y || sqrDist < squared_Distance){
                                saver = 0;
                        }
                        checker++;
                }
                pcenterList[numberPic-1] = ptc;
        }
        ccolor=CV_RGB(255,255,0);
        cvCircle(imcol,ptc,7,ccolor,1);

        fprintf(fp,"Probability of presence = %5.4f\n",prob);

        if(saver==1){
            fprintf(fp,"Centerpoint  x = %d, y = %d\n\n",ptc.x,ptc.y);
            cvSaveImage(fileName,imcol);
            globalSaver= 1;
        } else {
            fprintf(fp,"Matching redundant. Coordinates too close to a ready existing view!\n\n");
        }
    }
}

/**
This function performs an affine transformation to a vector.
*/
Matrice Dialog::Affine(Matrice vect, Matrice rot, Matrice transl)
{
    return rot*vect+transl;
}

/**
This function computes a similarity transform between the original points and the matched points.
*/
Matrice Dialog::SimilarityParamApprox(Match * tabmatch, int nbmatch, int maxdim, double * perr, int *pnbmatchgood)
{
    Matrice *a, *u, b(4,4),param(4,1,0),p1(2,1),p2(2,1),q1(2,1),q2(2,1),empty(0,0,0);
    int i, j;
    int maxindex;
    float maxerror=1, maxdtheta, maxrscale, maxdpos;

    int nbmatchgood=nbmatch;
    if(perr!=NULL)*perr=1e99;

    /* count good matches*/
    for(i=0;i<nbmatch;i++){tabmatch[i].id=1;}
    while(maxerror>0){

        a=new Matrice(2*nbmatchgood,4,0);
        u=new Matrice(2*nbmatchgood,1,0);

        j=0;
        for(i=0;i<nbmatch;i++){
            if(tabmatch[i].id!=0){
                if(2*j+1>=2*nbmatchgood){
                    cout<<"Error in ST"<<endl;
                }
                // First create the matrix A and u
                a->set_val(2*j,0,tabmatch[i].x1);
                a->set_val(2*j,1,-tabmatch[i].y1);
                a->set_val(2*j,2,1);
                a->set_val(2*j,3,0);
                a->set_val(2*j+1,0,tabmatch[i].y1);
                a->set_val(2*j+1,1,tabmatch[i].x1);
                a->set_val(2*j+1,2,0);
                a->set_val(2*j+1,3,1);
                u->set_val(2*j,0,tabmatch[i].x2);
                u->set_val(2*j+1,0,tabmatch[i].y2);
                j++;
            }
        }
        if(nbmatchgood<nb_match_min_ST){
            if(pnbmatchgood!=NULL)*pnbmatchgood=0;
            return empty;
        }
        b=a->t()*(*a);

       try
        {
            param = (b.inverse()*a->t())*(*u);
        }
        catch(Erreur &tmp)
        {
        
	}

        /// To verify the computed model, the error between the original points and the matched points according to the transformation is computed.
        // Test if a match fits the model
        maxerror=0;
        float theta=atan2(param.get_val(1,0),param.get_val(0,0));
        float logscale=sqrt(SQUARE(param.get_val(1,0),param.get_val(0,0)));
        logscale=log(logscale);

        for (i=0;i<nbmatch;i++){

            if(tabmatch[i].id!=0){
                p1.set_val(0,0,tabmatch[i].x1);
                p1.set_val(1,0,tabmatch[i].y1);
                p2=Similarity(p1,param);
                /* Compute error for each match*/

                float dx=tabmatch[i].x2-p2.get_val(0,0);
                float dy=tabmatch[i].y2-p2.get_val(1,0);

                float dtheta=abs(Set2RangePi(theta-(tabmatch[i].ori2-tabmatch[i].ori1)));
                float rscale=abs(logscale-log(tabmatch[i].scale2/tabmatch[i].scale1));

                //Each match must agree within  orientation, scale and position
                if(dtheta>agree_ori_ST||rscale>agree_scale_ST||dx*dx+dy*dy>agree_pos_ST*agree_pos_ST*maxdim*maxdim){
                    //                if(dx*dx+dy*dy>agreePos*agreePos*sqSizeOfProj){
                    float tmp=dtheta/agree_ori_ST+rscale/agree_scale_ST+sqrt(dx*dx+dy*dy)/agree_pos_ST/maxdim;
                    if (tmp>maxerror){

                        /* maxerror is the sum of weighted errors on position angle and scale*/
                        maxerror=tmp;
                        maxdtheta=dtheta/agree_ori_ST;
                        maxrscale=rscale/agree_scale_ST;
                        maxdpos=sqrt(dx*dx+dy*dy)/agree_pos_ST/maxdim;
                        maxindex=i;
                    }
                }
            }
        }//end for
        if(maxerror>0){
            tabmatch[maxindex].id=0;
            nbmatchgood-=1;
            if(nbmatchgood<nb_match_min_ST){
                if(pnbmatchgood!=NULL)*pnbmatchgood=0;
                return empty;
            }
        }
    }

    //Compute transformation error
    Matrice *c=new Matrice(2*nbmatchgood,1);
    *c=((*a)*param)-(*u);
    float err=sqrt(2*c->sqnorm()/(2*nbmatchgood-4));
    delete(c);
    if (perr!=NULL)*perr=err;

    delete(a);
    delete(u);

    if(pnbmatchgood!=NULL)
    *pnbmatchgood=nbmatchgood;
    if(nbmatchgood<nb_match_min_ST) return empty;
    else return param;
}

/**
This function applies a transformation (affine or similarity) 'param' to a given vector 'vect' according to the siye of the transformation matrix.
*/
Matrice Dialog::Transform(Matrice vect, Matrice param)
{
    if(param.size_m()==6 && param.size_n()==1){
        return Affine(vect, param);
    }
    else if(param.size_m()==4 && param.size_n()==1){
            return Similarity(vect, param);
        }
    else {
        cout<<"Wrong size of param in transform"<<endl;exit(1);
    }
}

/**
This function applies a similarity transform 'param' to the vector 'vect'.
*/
Matrice Dialog::Similarity(Matrice vect, Matrice param)
{
    if(param.size_m()!=4||param.size_n()!=1){
    }
    Matrice rot(2,2,0), transl(2,1,0);
    rot.set_val(0,0,param.get_val(0,0));
    rot.set_val(0,1,-param.get_val(1,0));
    rot.set_val(1,0,param.get_val(1,0));
    rot.set_val(1,1,param.get_val(0,0));
    transl.set_val(0,0,param.get_val(2,0));
    transl.set_val(1,0,param.get_val(3,0));

    return Affine(vect,rot,transl);
}

/**
This function applies a affine transform 'param' to the vector 'vect'.
*/
Matrice Dialog::Affine(Matrice vect, Matrice param)
{
    if(param.size_m()!=6 || param.size_n()!=1){
    }
    Matrice rot(2,2,0), transl(2,1,0);
    rot.set_val(0,0,param.get_val(0,0));
    rot.set_val(0,1,param.get_val(1,0));
    rot.set_val(1,0,param.get_val(2,0));
    rot.set_val(1,1,param.get_val(3,0));
    transl.set_val(0,0,param.get_val(4,0));
    transl.set_val(1,0,param.get_val(5,0));

    return Affine(vect,rot,transl);
}

double Dialog::Set2Range2Pi(double angle)
{
    double a=angle;
   while(a<0)a+=2*PI;
    while(a>=2*PI)a-=2*PI;
    return a;
}

double Dialog::Set2RangePi(double angle)
{
    double a=angle;
    while(a<-PI)a+=2*PI;
    while(a>=PI)a-=2*PI;
    return a;

}

void Dialog::clearClicked()
{
    bigEditor->clear();
}

void Dialog::ReadConfig()
{
    ifstream pf;
    pf.open(INIFILE);

    
    if(readINI ("distance_ratio", distance_ratio, pf)==false || distance_ratio==0)
    {
	cout<<"Error in "<<INIFILE<<" : distance_ratio"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 : distance_ratio").arg(INIFILE)); //You will never see this ...
	exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start
    //param for hough transform
    if(readINI ("division_in_position_x", x_div_HT, pf)==0|| x_div_HT==0)
    {
	cout<<"Error in "<<INIFILE<<" : division_in_position_x"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 division_in_position_x").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start

    if(readINI ("division_in_position_y", y_div_HT, pf)==0 || y_div_HT==0)
    {
	cout<<"Error in "<<INIFILE<<" : division_in_position_y"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 division_in_position_y").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start

    if(readINI ("division_in_angle", theta_div_HT, pf)==0 || theta_div_HT==0)
    {
	cout<<"Error in "<<INIFILE<<" : division_in_angle"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 division_in_angle").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start

    if(readINI ("division_in_scale", scale_div_HT, pf)==0|| scale_div_HT==0)
    {
	cout<<"Error in "<<INIFILE<<" : division_in_scale"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 division_in_scale").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start

    if(readINI ("scale_factor", scale_factor, pf)==0||scale_factor==0)
    {
	cout<<"Error in "<<INIFILE<<" : scale_factor"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 scale_factor").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start

    // param for affine transform

    if(readINI ("agree_in_position", agree_pos_AT, pf)==0 || agree_pos_AT==0)
    {
	cout<<"Error in "<<INIFILE<<" : agree_in_position"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 agree_in_position").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start

    if(readINI ("nb_match_min", nb_match_min_AT, pf)==0 || nb_match_min_AT==0)
    {
	cout<<"Error in "<<INIFILE<<" : nb_match_min"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 nb_match_min").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start

    // param for similarity transform
    if(readINI ("agree_in_position", agree_pos_ST, pf)==0 || agree_pos_ST==0)
    {
	cout<<"Error in "<<INIFILE<<" : agree_in_position"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 agree_in_position").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start

    if(readINI ("agree_in_angle", agree_ori_ST, pf)==0 || agree_ori_ST==0)
    {
	cout<<"Error in "<<INIFILE<<" : agree_in_angle"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 agree_in_angle").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start

    if(readINI ("agree_in_scale", agree_scale_ST, pf)==0 || agree_scale_ST==0)
    {
	cout<<"Error in "<<INIFILE<<" : agree_in_scale"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 agree_in_scale").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start

    if(readINI ("nb_match_min", nb_match_min_ST, pf)==0 || nb_match_min_ST==0)
    {
	cout<<"Error in "<<INIFILE<<" : nb_match_min"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 nb_match_min").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start

    // param for display

    if(readINI ("border_threshold", border_threshold, pf)==0 || border_threshold==0)
    {
	cout<<"Error in "<<INIFILE<<" : border_threshold"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 border_threshold").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start
    if(readINI ("squared_Distance", squared_Distance, pf)==0 || squared_Distance==0)
    {
	cout<<"Error in "<<INIFILE<<" : squared_Distance"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 squared_Distance").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start
    if(readINI ("contCaptureTimer", contCaptureTimer, pf)==0 || contCaptureTimer==0)
    {
	cout<<"Error in "<<INIFILE<<" : contCaptureTimer"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 contCaptureTimer").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start

    if(readINI ("clustering_threshold", clustering_threshold, pf)==0 || clustering_threshold==0)
    {
	cout<<"Error in "<<INIFILE<<" : clustering_threshold"<<endl; //stderr output.
	fflush(NULL);
        bigEditor->append(tr("Error in %1 clustering_threshold").arg(INIFILE)); exit(1);
    }  pf.seekg(ios_base::beg); //Get the file back to start
    // Keeps param in good range
    agree_scale_ST=log(agree_scale_ST);
    agree_ori_ST=agree_ori_ST*PI/180;
    theta_div_HT=RANGE(theta_div_HT,1,ThetaDivMax);
    scale_div_HT=RANGE(scale_div_HT,1,ScaleDivMax);
    x_div_HT=RANGE(x_div_HT,1,XDivMax);
    y_div_HT=RANGE(y_div_HT,1,YDivMax);

    bigEditor->append(tr(""));
    bigEditor->append(tr("distance_ratio: %1").arg(distance_ratio));
    bigEditor->append(tr("x_div_HT: %1").arg(x_div_HT));
    bigEditor->append(tr("y_div_HT: %1").arg(y_div_HT));
    bigEditor->append(tr("theta_div_HT: %1").arg(theta_div_HT));
    bigEditor->append(tr("scale_div_HT: %1").arg(scale_div_HT));
    bigEditor->append(tr("agree_pos_AT: %1").arg(agree_pos_AT));
    bigEditor->append(tr("agree_pos_ST: %1").arg(agree_pos_ST));
    bigEditor->append(tr("agree_ori_ST: %1").arg(agree_ori_ST));
    bigEditor->append(tr("clustering_threshold: %1").arg(clustering_threshold));
    bigEditor->append(tr("agree_scale_ST: %1").arg(agree_scale_ST));
    bigEditor->append(tr("border_threshold: %1").arg(border_threshold));
    bigEditor->append(tr("squared_Distance: %1").arg(squared_Distance));
    bigEditor->append(tr("contCaptureTimer: %1").arg(contCaptureTimer));

}
