/************************************************************************
Demo software: Invariant keypoint matching.
Author: David Lowe

defs.h:
This file contains the headers for a sample program to read images and
  keypoints, then perform simple keypoint matching.
*************************************************************************/

#ifndef DEFS_ONLY_ONCE_PLEASE_8797
#define DEFS_ONLY_ONCE_PLEASE_8797

/* From the standard C libaray: */
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include "highgui.h"
#define OS_WINDOWS
#define DISPLAY

/*------------------------------ Constants ---------------------------------*/

#define ThetaDivMax 12
#define ScaleDivMax 10
#define XDivMax 10
#define YDivMax 10
#define NB_MAX_TEMPLATES 300


#define PI 3.14159265
#define ClWhite 255
#define ClBlack 0
#define ClGray 127
/*For windows : */
#define CMDMAKE "makekeyfile "
#define INIFILE "config.ini"

/*------------------------------ Macros  ---------------------------------*/

#define ABS(x)    (((x) > 0) ? (x) : (-(x)))
#define MAXOF2(x,y)  (((x) > (y)) ? (x) : (y))
#define MINOF2(x,y)  (((x) < (y)) ? (x) : (y))
#define ROUND(x)  ((int)(x+0.5))
#define RANGE(x,a,b) (MINOF2(MAXOF2(x,a),b))
#define PRINT(x) cout<<"x"<<x<<endl;
#define SQUARE(x,y) (x)*(x)+(y)*(y)

#ifdef OS_LINUX
#define GET_TIME_MS
#endif
#ifdef OS_WINDOWS
#define GET_TIME_MS
#endif

/*---------------------------- Structures --------------------------------*/

/* Data structure for a keypoint.  Lists of keypoints are linked
   by the "next" field.*/

typedef struct KeypointSt {
  int degree;
  char name[50];
  IplImage *im;
  float row, col;                   /* Subpixel location of keypoint. */
  float scale, ori;                 /* Scale and orientation (range [-PI,PI]) */
  int nbim;                          /* Image number */
  char flag;                         /* Flag for use in program */
  float sq_dist_thres;          /// Default match threshold
  unsigned char *descrip;   /* Vector of descriptor values */
  struct KeypointSt *next;    /* Pointer to next keypoint in list. */
  struct KeypointSt *next2;  /* Pointer to next keypoint in list. */
  struct KeypointSt *match; /* Pointer to matched point. */
} *Keypoint;

/*Datastruct of a match : 2 keypoint coordinates*/
typedef struct MatchSt {
    int id;
    float x1;
    float y1;
    float ori1;
    float scale1;
    float x2;
    float y2;
    float ori2;
    float scale2;
} Match;

/*Datastruct of a indexSt : Linked list of all the cells indices of the coherent keypoints*/
typedef struct IndexSt {
    int i1;
    int i2;
    int i3;
    int i4;
    struct IndexSt *next;
} IndexList;

/*Datastruct of a NamelistSt : Stores filenames of the model views (top level)*/
typedef struct NamelistSt {
    char name[50];
    int id;
    struct NamelistSt *next;
} Namelist;

#endif
