/***************************************************************************
 *   Copyright (C) 2007 by ASL-ETH and IIIA-CSIC                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "ransac.h"

#define RANSAC_SUCCESS_PROBABILITY 0.9
#define RANSAC_EST_INLIER_RATE 0.4
#define RANSAC_THRESHOLD 0.5

void generate3random(int* rand_points,int n_points)
{
    bool repeated=false;
    for(int i=0;i<3;i++)
    {
        do
        {
            rand_points[i]=rand()%n_points;
            repeated=false;
            switch(i)
            {
                case 1:
                    if(rand_points[0]==rand_points[i])
                        repeated=true;
                break;
                case 2:
                    if(rand_points[0]==rand_points[i] || rand_points[1]==rand_points[i])
                        repeated=true;
            }
        }while(repeated);
    }
}

void coutmatrix(CvMat* M)
{
        cout<<endl;
        for(int i=0;i< M->rows;i++)
        {
            for(int j=0;j< M->cols;j++)
            {
                cout<<M->data.fl[M->cols*i+j]<<" ";
            }
            cout<<endl;
        }
        cout<<endl;
}

void coutmatrix(CvMat* M, ofstream& pf)
{
        pf<<endl;
        for(int i=0;i< M->rows;i++)
        {
            for(int j=0;j< M->cols;j++)
            {
                pf<<M->data.fl[M->cols*i+j]<<" ";
            }
            pf<<endl;
        }
        pf<<endl;
}

float median(const vector<float>* resid)
{

	if(!resid->empty()){
		int size=resid->size();
		vector<float> vect;
		for(int i=0;i<size;i++)
		{
			vect.push_back((*resid)[i]);
		}
		sort(vect.begin(),vect.end());
		if(size%2)
		{
			//Case uneven
			return (vect)[(int)(size/2)];

		}
		else
		{
			//Case even
			return (((vect)[size/2]+(vect)[(size/2)-1])/2.0);

		}
	}
	else return 0.0;
}

Matrice RansacAffine(Match* kp_list,int n_points, vector<int> &inliers, float &error)
{
    // Number of iterations
    int MAX_iterations;
    float best_median=-1;
    float m_median;
    error=0;

    Matrice return_affine(6,1,0);

    CvMat* A=cvCreateMat(6,6,CV_32FC1);
    CvMat* b=cvCreateMat(6,1,CV_32FC1);
    CvMat* affi=cvCreateMat(6,1,CV_32FC1);
    CvMat* affi2=cvCreateMat(3,3,CV_32FC1);
    CvMat* pts=cvCreateMat(3,n_points,CV_32FC1);
    CvMat* pts_ref=NULL;
    CvMat* ptsb_ref=NULL;
    CvMat* ptso_a=cvCreateMat(3,n_points,CV_32FC1);
    CvMat* ptso_b=cvCreateMat(3,n_points,CV_32FC1);
    CvMat* best_affi=cvCreateMat(3,3,CV_32FC1);
    CvMat * T1=NULL, * T2=NULL;
    MAX_iterations = (int) ceil(log(1-RANSAC_SUCCESS_PROBABILITY)/log(1-pow(RANSAC_EST_INLIER_RATE,3)));
    MAX_iterations = min(MAX_iterations,1000);

    //Point Normalization
    normalize_pts(kp_list,&ptso_a,&ptso_b,&T1,&T2);
    cvInvert(T2,T2,CV_SVD);

    if(n_points<16 && 0)
    {
        // TODO: +++ if n_points=15 the number of tests to try all possibilities is 455, which is assumible.

    }
    else
    {

        srand( (unsigned int)time( NULL )); //seed for random
        int rand_points[3];
        int rand_points_selected[3];

        cout<<"MAX Iterations = "<<MAX_iterations<<endl;

        for(int k=0;k<MAX_iterations;k++)
        {


        // Random number generation
            generate3random(rand_points,n_points);

        //Matrice preparation
            A->data.fl[0]=ptso_a->data.fl[rand_points[0]];
            A->data.fl[1]=0;
            A->data.fl[2]=ptso_a->data.fl[rand_points[0]+n_points];
            A->data.fl[3]=0;
            A->data.fl[4]=1;
            A->data.fl[5]=0;

            A->data.fl[6]=0;
            A->data.fl[7]=ptso_a->data.fl[rand_points[0]];
            A->data.fl[8]=0;
            A->data.fl[9]=ptso_a->data.fl[rand_points[0]+n_points];
            A->data.fl[10]=0;
            A->data.fl[11]=1;

            b->data.fl[0]=ptso_b->data.fl[rand_points[0]];
            b->data.fl[1]=ptso_b->data.fl[rand_points[0]+n_points];


            A->data.fl[12]=ptso_a->data.fl[rand_points[1]];
            A->data.fl[13]=0;
            A->data.fl[14]=ptso_a->data.fl[rand_points[1]+n_points];
            A->data.fl[15]=0;
            A->data.fl[16]=1;
            A->data.fl[17]=0;

            A->data.fl[18]=0;
            A->data.fl[19]=ptso_a->data.fl[rand_points[1]];
	    A->data.fl[20]=0;
            A->data.fl[21]=ptso_a->data.fl[rand_points[1]+n_points];
            A->data.fl[22]=0;
            A->data.fl[23]=1;

            b->data.fl[2]=ptso_b->data.fl[rand_points[1]];
            b->data.fl[3]=ptso_b->data.fl[rand_points[1]+n_points];


            A->data.fl[24]=ptso_a->data.fl[rand_points[2]];
            A->data.fl[25]=0;
            A->data.fl[26]=ptso_a->data.fl[rand_points[2]+n_points];
            A->data.fl[27]=0;
            A->data.fl[28]=1;
            A->data.fl[29]=0;

            A->data.fl[30]=0;
            A->data.fl[31]=ptso_a->data.fl[rand_points[2]];
            A->data.fl[32]=0;
            A->data.fl[33]=ptso_a->data.fl[rand_points[2]+n_points];
            A->data.fl[34]=0;
            A->data.fl[35]=1;

            b->data.fl[4]=ptso_b->data.fl[rand_points[2]];
            b->data.fl[5]=ptso_b->data.fl[rand_points[2]+n_points];

        //Computation of affine transform
            cvInvert(A,A,CV_SVD);
            cvMatMul(A,b,affi);

            for(int i1=0;i1<3;i1++)
            {
                affi2->data.fl[i1]=affi->data.fl[i1*2];
                affi2->data.fl[i1+3]=affi->data.fl[i1*2+1];
            }
            affi2->data.fl[6]=0;affi2->data.fl[7]=0;affi2->data.fl[8]=1;

            cvMatMul(affi2,ptso_a,pts);

            vector<float> resid;
            for(int i3=0;i3<n_points;i3++)
            {
                resid.push_back(sqrt(pow((pts->data.fl[i3]-ptso_b->data.fl[i3]),2)+
                                    pow((pts->data.fl[i3+n_points]-ptso_b->data.fl[i3+n_points]),2))
                                    );
            }

/* SIMPLE affine transf. score */
            if(best_median==-1)
            {
                best_median=median(&resid);
                rand_points_selected[0]=rand_points[0];
                rand_points_selected[1]=rand_points[1];
                rand_points_selected[2]=rand_points[2];

                cvCopy( affi2, best_affi );
	    }
	    else
	    {
            	m_median=median(&resid);
            	if(m_median<best_median)
            	{
	                best_median=m_median;
                	rand_points_selected[0]=rand_points[0];
                	rand_points_selected[1]=rand_points[1];
                	rand_points_selected[2]=rand_points[2];

	                cvCopy( affi2, best_affi );
            	}
	    }

            // +++ MISSING +++ A condition to break the loop if a certain resid threshold is meet.
        } // **** END OF MAIN LOOP ****

// Best RANDOM affine matrix found --> best_affi. Now it's time to REFINE it.

        cvMatMul(best_affi,ptso_a,pts);

	vector<float> resid;
	error=0;
        for(int i3=0;i3<n_points;i3++)
        {
            resid.push_back(sqrt(pow((pts->data.fl[i3]-ptso_b->data.fl[i3]),2)+
                        pow((pts->data.fl[i3+n_points]-ptso_b->data.fl[i3+n_points]),2))
                        );
            if(resid[i3]<RANSAC_THRESHOLD)
            {
                inliers.push_back(i3);
                error+=resid[i3];
            }
        }
	pts_ref=cvCreateMat(2*inliers.size(),6,CV_32FC1);
	ptsb_ref=cvCreateMat(2*inliers.size(),1,CV_32FC1);

	for(unsigned int hh=0;hh<inliers.size();hh++)
	{
		//Matrice preparation
		pts_ref->data.fl[pts_ref->cols*2*hh+0]=ptso_a->data.fl[(inliers[hh])];
		pts_ref->data.fl[pts_ref->cols*2*hh+1]=0;
 		pts_ref->data.fl[pts_ref->cols*2*hh+2]=ptso_a->data.fl[(inliers[hh])+n_points];
		pts_ref->data.fl[pts_ref->cols*2*hh+3]=0;
		pts_ref->data.fl[pts_ref->cols*2*hh+4]=1;
		pts_ref->data.fl[pts_ref->cols*2*hh+5]=0;
		pts_ref->data.fl[pts_ref->cols*2*hh+6]=0;
		pts_ref->data.fl[pts_ref->cols*2*hh+7]=ptso_a->data.fl[(inliers[hh])]; 
		pts_ref->data.fl[pts_ref->cols*2*hh+8]=0;
		pts_ref->data.fl[pts_ref->cols*2*hh+9]=ptso_a->data.fl[(inliers[hh])+n_points];
		pts_ref->data.fl[pts_ref->cols*2*hh+10]=0;
		pts_ref->data.fl[pts_ref->cols*2*hh+11]=1;
 
		ptsb_ref->data.fl[2*hh+0]=ptso_b->data.fl[(inliers[hh])];
 		ptsb_ref->data.fl[2*hh+1]=ptso_b->data.fl[(inliers[hh])+n_points];
	}
	coutmatrix(pts_ref);
	coutmatrix(ptsb_ref);

	// +++ MISSING +++ recomputation -> result of RANSAC is a rough estimate for next step!

        //copy result to Matrice

        cvGEMM( T2, best_affi, 1, NULL, 1, best_affi, 0 ); //T2 is already inverted
	cvGEMM( best_affi, T1, 1, NULL, 1, best_affi, 0 );
coutmatrix(best_affi);
cout<<"error: "<<error<<endl;

        return_affine.set_val(0,0,best_affi->data.fl[0]);
        return_affine.set_val(1,0,best_affi->data.fl[3]);
        return_affine.set_val(2,0,best_affi->data.fl[6]);
        return_affine.set_val(3,0,best_affi->data.fl[1]);
        return_affine.set_val(4,0,best_affi->data.fl[4]);
        return_affine.set_val(5,0,best_affi->data.fl[7]);
    }

cvReleaseMat(&A);
cvReleaseMat(&b);
cvReleaseMat(&affi);
cvReleaseMat(&affi2);
cvReleaseMat(&best_affi);
cvReleaseMat(&pts);
cvReleaseMat(&ptso_a);
cvReleaseMat(&ptso_b);
cvReleaseMat(&pts_ref);
cvReleaseMat(&ptsb_ref);
if(T1!=NULL)
	cvReleaseMat(&T1);
if(T2!=NULL)
	cvReleaseMat(&T2);


return return_affine;
}





void normalize_pts(Match* kp_list,CvMat **ptso_a,CvMat **ptso_b,CvMat **T1,CvMat **T2)
{
    int n_points=(*ptso_a)->cols;

    CvMat* ptsaux_a=cvCreateMat(3,n_points,CV_32FC1);
    CvMat* ptsaux_b=cvCreateMat(3,n_points,CV_32FC1);

    for(int i=0;i<n_points;i++)
    {
        ptsaux_a->data.fl[i]=kp_list[i].x1;
        ptsaux_a->data.fl[i+n_points]=kp_list[i].y1;
        ptsaux_a->data.fl[i+(n_points*2)]=1.0;

        ptsaux_b->data.fl[i]=kp_list[i].x2;
        ptsaux_b->data.fl[i+n_points]=kp_list[i].y2;
        ptsaux_b->data.fl[i+(n_points*2)]=1.0;
    }

    normalise2dpts(ptsaux_a, ptso_a,T1);
    normalise2dpts(ptsaux_b, ptso_b,T2);
    
    cvReleaseMat(&ptsaux_a);
    cvReleaseMat(&ptsaux_b);
}


void normalise2dpts(CvMat *pts, CvMat **newpts, CvMat **T)
{
   CV_FUNCNAME("normalise2dpts");

   __BEGIN__;

   float mean_x = 0, mean_y = 0, meandist = 0, scale;
   CvSize mida, mida_new;
   if (pts == NULL)
       CV_ERROR(CV_StsNullPtr, "First parameter cannot be a NULL pointer");

   mida = cvGetSize(pts);
   if (*newpts == NULL) *newpts = cvCreateMat(mida.height, mida.width, cvGetElemType(pts));
   if (*T == NULL) *T = cvCreateMat(3, 3, cvGetElemType(pts));
   mida_new = cvGetSize(*newpts);
   if ((mida.height != 3) && (mida.width != 3))
       CV_ERROR(CV_StsUnmatchedSizes, "Matrix must be 3xN or Nx3");
   if ((mida.height != mida_new.height) || (mida.width != mida_new.width))
       CV_ERROR(CV_StsUnmatchedSizes, "Matrix dimensions must agree");
   if (mida.width == 3)
   {
       for (int i = 0; i < mida.height; ++i)
       {
           mean_x += cvmGet(pts, i, 0);
           mean_y += cvmGet(pts, i, 1);
       }
       mean_x = mean_x / (float)mida.height;
       mean_y = mean_y / (float)mida.height;
       for (int i = 0; i < mida.height; ++i) //Points are centered
       {
           cvmSet(*newpts, i, 0, cvmGet(pts, i, 0) - mean_x);
           cvmSet(*newpts, i, 1, cvmGet(pts, i, 1) - mean_y);
           meandist += sqrt(cvmGet(*newpts, i, 0) * cvmGet(*newpts, i, 0) + cvmGet(*newpts, i, 1) * cvmGet(*newpts, i, 1));
       }
       meandist = meandist / (float)mida.height; //The scale is found
       scale = sqrt(2) / meandist;
       cvmSet(*T, 0, 0, scale);
       cvmSet(*T, 0, 1, 0);
       cvmSet(*T, 0, 2, -scale * mean_x);
       cvmSet(*T, 1, 0, 0);
       cvmSet(*T, 1, 1, scale);
       cvmSet(*T, 1, 2, -scale * mean_y);
       cvmSet(*T, 2, 0, 0);
       cvmSet(*T, 2, 1, 0);
       cvmSet(*T, 2, 2, 1);
       cvGEMM(pts, *T, 1, 0, 0, *newpts, CV_GEMM_B_T);
   }
   else
   {
       for (int i = 0; i < mida.width; ++i)
       {
           mean_x += cvmGet(pts, 0, i);
           mean_y += cvmGet(pts, 1, i);
       }
       mean_x = mean_x / (float)mida.width;
       mean_y = mean_y / (float)mida.width;
       for (int i = 0; i < mida.width; ++i)
       {
           cvmSet(*newpts, 0, i, cvmGet(pts, 0, i) - mean_x);
           cvmSet(*newpts, 1, i, cvmGet(pts, 1, i) - mean_y);
           meandist += sqrt(cvmGet(*newpts, 0, i) * cvmGet(*newpts, 0, i) + cvmGet(*newpts, 1, i) * cvmGet(*newpts, 1, i));
       }
       meandist = meandist / (float)mida.width;
       scale = sqrt(2) / meandist;
       cvmSet(*T, 0, 0, scale);
       cvmSet(*T, 0, 1, 0);
       cvmSet(*T, 0, 2, -scale * mean_x);
       cvmSet(*T, 1, 0, 0);
       cvmSet(*T, 1, 1, scale);
       cvmSet(*T, 1, 2, -scale * mean_y);
       cvmSet(*T, 2, 0, 0);
       cvmSet(*T, 2, 1, 0);
       cvmSet(*T, 2, 2, 1);
       cvGEMM(*T, pts, 1, 0, 0, *newpts, 0);
   }

   __END__;
}
